//    WeIRCd 0.4 IRC server software - main file
//    Copyright (C) 2007, 2008 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


//----------------------------------------------------------//
//     ///////////////////////////////////////////          //
//     //////       COMPILE OPTIONS        ///////          //
//     ///////////////////////////////////////////          //
//      f e e l    f r e e    t o    m o d i f y            //
//----------------------------------------------------------//

#define SERVICES_LOGINTIMEOUT 30
#define SERVICES_ENABLED
//#define SERVICES_ENFORCEPROTECT

#define COMPILEOPTION_MAXGLINES 60 //more doesn't really make sense as they need to be few enough for a complete list of them being printed out when doing /gline list

#define COMPILEOPTION_PINGFREQUENCY 120

#define COMPILEOPTION_MAXUSERS 10000 //don't hesitate to put a big number here, only 4 bytes for each unused user are lost for an empty pointer
#define COMPILEOPTION_MAXCHANS 2000 //same goes for chans
#define COMPILEOPTION_MAXCHANSPERUSER 20 //bigger is possible here aswell, but probably doesn't make too much sense

#define COMPILEOPTION_IPSTORAGESIZE 50000 //should be (much) bigger than max users

//maximum len of different string values, e.g. the user's host, the nick etc. (it's better not to change this too much)
#define COMPILEOPTION_MAXHOSTLEN 60
#define COMPILEOPTION_MAXNICKLEN 25
#define COMPILEOPTION_MAXAWAYLEN 200
#define COMPILEOPTION_MAXREALNAMELEN 50
#define COMPILEOPTION_MAXUSERIDLEN 10

//----------------------------------------------------------//
//     ///////////////////////////////////////////          //
//     //////    END of compile options    ///////          //
//     ///////////////////////////////////////////          //
//                                                          //
//----------------------------------------------------------//

#include "weircd.h"

int main() {
	signal(SIGTERM, sigterm);
	if (weircdInit(version_detail) < 0) {return -1;}
	if (getuid() == 0 || geteuid() == 0) {
		printf("%s Running as root - aborting...\n\n",startupprefix);
		printf("------ DO >NOT< RUN WEIRCD AS ROOT USER ------\n");
		printf("        This is dangerous. Please create\n");
		printf("        a user account that has only the\n");
		printf("        limited privilegues that an ircd\n");
		printf("        needs.\n");
		printf("----------------------------------------------\n");
		return -1;
	}
	if (readconfig(0) < 0) {
		printf("%s    > Error processing config file, Cancelling.\n",startupprefixsub);
		return -1;
	}
	printf("%s Loading glines...\n",startupprefix);
	if (readglines() < 0) {
		printf("%s    > Error processing gline file, Cancelling.\n",startupprefixsub);
		return -1;
	}
	struct sigaction signalst;
	if (sigaction(SIGPIPE, NULL, &signalst) == 0) {
		signalst.sa_handler = SIG_IGN;
		sigaction(SIGPIPE, &signalst, NULL);
	}else{
		printf("%s Failed to mask SIGPIPE signal handler\n",startupprefix);
		return -1;
	}
	socketsStartup(currentoptions.bindip,currentoptions.bindport);
#ifndef NOFORK
	int forking = forkIntoBackground();
	if (forking < 0) {
		if (forking < -1) {
			printf("ERROR: Forking failed.\n");
			return -1;
		}else{
			return 0;
		}
	}
	if (setsid() < 0) {return -1;}
	forking = forkIntoBackground();
	if (forking < 0) {
		if (forking < -1) {
			return -1;
		}else{
			return 0;
		}
	}
	umask(0);
#endif
	return weircdMainLoop();
}
